from functools import wraps

events = []

def add_channel(*args, **kwargs):
    pusher_data = {}

    if args and type(args[0]) == str:
        pusher_data['channel_name'] = args[0]
    
    for key, value in kwargs.items():
        if key not in pusher_data:
            pusher_data[key] = value
    
    if 'channel_name' not in pusher_data:
        raise TypeError('add_channel() missing 1 required positional argument: \'channel_name\'')

    
    def wrapper(function):

        @wraps(function)
        def _wrapper():
            if 'event_name' not in pusher_data:
                pusher_data['event_name'] = function.__name__


            events.append(
                {
                    'channel_name': pusher_data['channel_name'],
                    'event_name': pusher_data['event_name'],
                    'function': function,
                }
            )

        return _wrapper()

    return wrapper


from django.core.management.base import BaseCommand
from dj_pysher import PysherApplication

class Command(BaseCommand):

    help = "Whatever you want to print here"

    def handle(self, *args, **options):
        app = PysherApplication()
        app.main()


import pysher
import sys
import time
# Add a logging handler so we can see the raw communication data
import logging
from django.conf import settings


root = logging.getLogger()
root.setLevel(logging.INFO)
ch = logging.StreamHandler(sys.stdout)
root.addHandler(ch)


class PysherApplication(object):
    appkey = settings.PUSHER_APP_KEY or ''
    secret = settings.PUSHER_SECRET or ''
    cluster = settings.PUSHER_CLUSTER or ''
    secure = settings.PUSHER_SECURE 

    def __init__(self):
        self.client = pysher.Pusher(key=self.appkey, secret=self.secret, cluster=self.cluster, secure=self.secure)


    def prepare_event(self, channel_name, event_name, function):
        def connect_handler(data):
            channel = self.client.subscribe(channel_name)
            channel.bind(event_name, function)

        self.client.connection.bind('pusher:connection_established', connect_handler)


    def prepare_events(self):
        self._autodiscover_events()
        from dj_pysher.tools import events
        for event in events:
            self.prepare_event(
                event['channel_name'],
                event['event_name'],
                event['function']
            )

        self.client.connect()

    def _autodiscover_events(self):
        for package in (lambda: settings.INSTALLED_APPS)():
            try:
                exec('from {}.events import *'.format(package))
                print('Loaded {}'.format(package))
            except Exception as ex:
                print(ex)



    def main(self):
        self.prepare_events()
        while True:
            # Do other things in the meantime here...
            time.sleep(1)



from pusher import pusher
from django.conf import settings


class SenderApplication(object):
    app_id = settings.PUSHER_APP_ID or ''
    appkey = settings.PUSHER_APP_KEY or ''
    secret = settings.PUSHER_SECRET or ''
    cluster = settings.PUSHER_CLUSTER or ''
    secure = settings.PUSHER_SECURE 

    def __init__(self):
        """
        Construtor
        :param channel: Canal
        :param event: Evento
        :param data: Dicionario
        """
        self.client = pusher.Pusher(
            app_id=self.app_id,
            key=self.appkey,
            secret=self.secret,
            cluster=self.cluster,
            ssl=self.secure
        )

    def send(self, channel, event, data):
        self.client.trigger(channel, event, data)

    def set_channel(self, channel):
        
        def func(event, data):
            self.client.trigger(channel, event, data)

        return func
